import 'package:flutter/cupertino.dart';
import 'package:nui_core/data/nui_ent.dart';
import 'package:nui_core/nui_core.dart';
import 'package:nui_database/models/nui_database_entity.dart';
import 'package:nui_database/models/nui_database_entity_field.dart';
import 'package:nui_database/models/nui_database_entity_mapper.dart';

enum NUILogLevel{
  DEBUG,
  TRACE,
  ERROR
}
extension NUILogLevelExt on NUILogLevel{
  int value(){
    switch(this){
      case NUILogLevel.DEBUG : return 1;
      case NUILogLevel.TRACE : return 2;
      case NUILogLevel.ERROR : return 3;
    }
  }
}

enum NUILogActionType{
  LOGIN,
  LOGOUT,
  VIEW,
  UPDATE,
  ADD,
  DELETE,
  EXPORT,
  GRANT,
  REVOKE,
  SUBMIT,
  APPROVE,
  REJECT,
  OTHER
}
NUILogActionType logActionTypeValueOf(int value){
  switch(value){
    case 1000: return NUILogActionType.LOGIN;
    case 1002: return NUILogActionType.LOGOUT;
    case 1004: return NUILogActionType.VIEW;
    case 1006: return NUILogActionType.UPDATE;
    case 1008: return NUILogActionType.ADD;
    case 1010: return NUILogActionType.DELETE;
    case 1012: return NUILogActionType.EXPORT;
    case 1014: return NUILogActionType.GRANT;
    case 1016: return NUILogActionType.REVOKE;
    case 1018: return NUILogActionType.SUBMIT;
    case 1020: return NUILogActionType.APPROVE;
    case 1022: return NUILogActionType.REJECT;
    default: return NUILogActionType.OTHER;
  }
}
extension NUILogActionTypeExt on NUILogActionType{
  int code(){
    switch(this){
      case NUILogActionType.LOGIN: return 1000;
      case NUILogActionType.LOGOUT: return 1002;
      case NUILogActionType.VIEW: return 1004;
      case NUILogActionType.UPDATE: return 1006;
      case NUILogActionType.ADD: return 1008;
      case NUILogActionType.DELETE: return 1010;
      case NUILogActionType.EXPORT: return 1012;
      case NUILogActionType.GRANT: return 1014;
      case NUILogActionType.REVOKE: return 1016;
      case NUILogActionType.SUBMIT: return 1018;
      case NUILogActionType.APPROVE: return 1020;
      case NUILogActionType.REJECT: return 1022;
      case NUILogActionType.OTHER: return 9999;
      default: return 9999;
    }
  }
}

enum NUILogValueType{
  AUDIT_TRAIL,
  CRASH,
  ERROR,
  WARNING,
  INFO,
  DEBUG,
  OTHER
}
NUILogValueType logValueTypeValueOf(int value){
  switch(value){
    case 1000: return NUILogValueType.AUDIT_TRAIL;
    case 1002: return NUILogValueType.CRASH;
    case 1004: return NUILogValueType.ERROR;
    case 1006: return NUILogValueType.WARNING;
    case 1008: return NUILogValueType.INFO;
    case 1010: return NUILogValueType.DEBUG;
    default: return NUILogValueType.OTHER;
  }
}
extension NUILogValueTypeExt on NUILogValueType{
  int code(){
    switch(this){
      case NUILogValueType.AUDIT_TRAIL: return 1000;
      case NUILogValueType.CRASH: return 1002;
      case NUILogValueType.ERROR: return 1004;
      case NUILogValueType.WARNING: return 1006;
      case NUILogValueType.INFO: return 1008;
      case NUILogValueType.DEBUG: return 1010;
      default: return 9999;
    }
  }
}

class NUILogContent extends NUIDBEntity<NUILogContent>{
  NUILogContent({
    this.id,
    this.action,
    this.error,
    this.logLevel,
    this.platform,
    @required this.description,
    this.lineNumber,
    this.affectFile,
    this.hardwareModel,
    this.osVersion,
    this.userId,
    this.happen,
    this.appVersion,
    this.module,
    this.subModule,
    this.bundleId,
    this.sessionId
  });

  NUILogContent.empty();
  String id;
  int action;
  int error;
  int logLevel;
  String platform;
  String description;
  int lineNumber;
  String affectFile;
  String hardwareModel;
  String osVersion;
  String userId;
  DateTime happen; //Date time
  String appVersion;
  String module;
  String subModule;
  String bundleId;
  String sessionId;

  Map<String, String> toLogMap(){
    return {
      "id": id,
      "action": action.toString(),
      "error": error.toString(),
      "logLevel": logLevel.toString(),
      "platform": platform,
      "description": description,
      "lineNumber": lineNumber.toString(),
      "affectFile": affectFile,
      "hardwareModel": hardwareModel,
      "osVersion": osVersion,
      "appVersion": appVersion,
      "module": module,
      "subModule": subModule
    };
  }

  @override
  NUIEntMapper<NUILogContent> mapper() => NUILogContentEntity();
}

class NUILogContentEntity extends NUIDBEntityMapper<NUILogContent>{
  @override
  List<NUIDBField> dbFields() {
    return [
      NUIDBField<NUILogContent>(name: "id", type: NUIEntType.STRING, setter: (data, value) => data.id = value, getter: (data) => data.id, primaryKey: true),
      NUIDBField<NUILogContent>(name: "action", type: NUIEntType.INTEGER, setter: (data, value) => data.action = value, getter: (data) => data.action),
      NUIDBField<NUILogContent>(name: "error", type: NUIEntType.INTEGER, setter: (data, value) => data.error = value, getter: (data) => data.error),
      NUIDBField<NUILogContent>(name: "loglevel", type: NUIEntType.INTEGER, setter: (data, value) => data.logLevel = value, getter: (data) => data.logLevel),
      NUIDBField<NUILogContent>(name: "platform", type: NUIEntType.STRING, setter: (data, value) => data.platform = value, getter: (data) => data.platform),
      NUIDBField<NUILogContent>(name: "description", type: NUIEntType.STRING, setter: (data, value) => data.description = value, getter: (data) => data.description),
      NUIDBField<NUILogContent>(name: "lineNumber", type: NUIEntType.INTEGER, setter: (data, value) => data.lineNumber = value, getter: (data) => data.lineNumber),
      NUIDBField<NUILogContent>(name: "affectFile", type: NUIEntType.STRING, setter: (data, value) => data.affectFile = value, getter: (data) => data.affectFile),
      NUIDBField<NUILogContent>(name: "hardwareModel", type: NUIEntType.STRING, setter: (data, value) => data.hardwareModel = value, getter: (data) => data.hardwareModel),
      NUIDBField<NUILogContent>(name: "osVersion", type: NUIEntType.STRING, setter: (data, value) => data.osVersion = value, getter: (data) => data.osVersion),
      NUIDBField<NUILogContent>(name: "userId", type: NUIEntType.STRING, setter: (data, value) => data.userId = value, getter: (data) => data.userId),
      NUIDBField<NUILogContent>(name: "happen", type: NUIEntType.DATE, setter: (data, value) => data.happen = asDateTime(value), getter: (data) => data.happen),
      NUIDBField<NUILogContent>(name: "appVersion", type: NUIEntType.STRING, setter: (data, value) => data.appVersion = value, getter: (data) => data.appVersion),
      NUIDBField<NUILogContent>(name: "module", type: NUIEntType.STRING, setter: (data, value) => data.module = value, getter: (data) => data.module),
      NUIDBField<NUILogContent>(name: "submodule", type: NUIEntType.STRING, setter: (data, value) => data.subModule = value, getter: (data) => data.subModule),
      NUIDBField<NUILogContent>(name: "bundleId", type: NUIEntType.STRING, setter: (data, value) => data.bundleId = value, getter: (data) => data.bundleId),
      NUIDBField<NUILogContent>(name: "sessionId", type: NUIEntType.STRING, setter: (data, value) => data.sessionId = value, getter: (data) => data.sessionId),
    ];
  }

  @override
  String entityName() => "NUILogContent";

  @override
  NUILogContent newInstance() => NUILogContent.empty();
}

class NUILogSetting extends NUIDBEntity<NUILogSetting>{
  NUILogSetting.empty();
  int logLevel;
  bool enable;

  @override
  NUIEntMapper<NUILogSetting> mapper() => NUILogSettingEntity();
}

class NUILogSettingEntity extends NUIDBEntityMapper<NUILogSetting>{
  @override
  List<NUIDBField> dbFields() {
    return [
      NUIDBField<NUILogSetting>(name: "logLevel", type: NUIEntType.INTEGER, setter: (data, value) => data.logLevel = value, getter: (data) => data.logLevel),
      NUIDBField<NUILogSetting>(name: "enable", type: NUIEntType.BOOLEAN, setter: (data, value) => data.enable = value, getter: (data) => data.enable),
    ];
  }

  @override
  String entityName() => "NUILogSetting";

  @override
  NUILogSetting newInstance() => NUILogSetting.empty();
}